# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any, Optional, TYPE_CHECKING

from azure.mgmt.core import AsyncARMPipelineClient
from msrest import Deserializer, Serializer

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential

from ._configuration import AlertsManagementClientConfiguration
from .operations import ActionRulesOperations
from .operations import Operations
from .operations import AlertsOperations
from .operations import SmartGroupsOperations
from .operations import SmartDetectorAlertRulesOperations
from .. import models


class AlertsManagementClient(object):
    """AlertsManagement Client.

    :ivar action_rules: ActionRulesOperations operations
    :vartype action_rules: azure.mgmt.alertsmanagement.aio.operations.ActionRulesOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.alertsmanagement.aio.operations.Operations
    :ivar alerts: AlertsOperations operations
    :vartype alerts: azure.mgmt.alertsmanagement.aio.operations.AlertsOperations
    :ivar smart_groups: SmartGroupsOperations operations
    :vartype smart_groups: azure.mgmt.alertsmanagement.aio.operations.SmartGroupsOperations
    :ivar smart_detector_alert_rules: SmartDetectorAlertRulesOperations operations
    :vartype smart_detector_alert_rules: azure.mgmt.alertsmanagement.aio.operations.SmartDetectorAlertRulesOperations
    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription.
    :type subscription_id: str
    :param str base_url: Service URL
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        if not base_url:
            base_url = 'https://management.azure.com'
        self._config = AlertsManagementClientConfiguration(credential, subscription_id, **kwargs)
        self._client = AsyncARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

        self.action_rules = ActionRulesOperations(
            self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(
            self._client, self._config, self._serialize, self._deserialize)
        self.alerts = AlertsOperations(
            self._client, self._config, self._serialize, self._deserialize)
        self.smart_groups = SmartGroupsOperations(
            self._client, self._config, self._serialize, self._deserialize)
        self.smart_detector_alert_rules = SmartDetectorAlertRulesOperations(
            self._client, self._config, self._serialize, self._deserialize)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "AlertsManagementClient":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details) -> None:
        await self._client.__aexit__(*exc_details)
